@@ -12,15 +12,16 @@ from collections import defaultdict
 
 from twitter.common.collections.orderedset import OrderedSet
 
-from pants.backend.core.tasks.task import Task
 from pants.backend.jvm.targets.annotation_processor import AnnotationProcessor
 from pants.backend.jvm.targets.scala_library import ScalaLibrary
-from pants.backend.jvm.tasks.jvm_tool_task_mixin import JvmToolTaskMixin
-from pants.backend.project_info.tasks.projectutils import get_jar_infos
+from pants.backend.jvm.tasks.classpath_products import ClasspathProducts
+from pants.backend.jvm.tasks.ivy_task_mixin import IvyTaskMixin
+from pants.backend.jvm.tasks.nailgun_task import NailgunTask
 from pants.base.build_environment import get_buildroot
 from pants.base.exceptions import TaskError
-from pants.base.source_root import SourceRoot
 from pants.binaries import binary_util
+from pants.build_graph.address import BuildFileAddress
+from pants.build_graph.resources import Resources
 from pants.util.dirutil import safe_mkdir, safe_walk
 
 
@@ -33,7 +34,6 @@ logger = logging.getLogger(__name__)
 # 2.) the target may be under development in which case it may not have sources yet - its pretty
 #     common to write a BUILD and ./pants idea the target inside to start development at which
 #     point there are no source files yet - and the developer intents to add them using the ide.
-
 def is_scala(target):
   return target.has_sources('.scala') or target.is_scala
 
@@ -42,10 +42,7 @@ def is_java(target):
   return target.has_sources('.java') or target.is_java
 
 
-class IdeGen(JvmToolTaskMixin, Task):
-
-  RESOURCES = "java-resource"
-  TEST_RESOURCES = "java-test-resource"
+class IdeGen(IvyTaskMixin, NailgunTask):
 
   @classmethod
   def register_options(cls, register):
@@ -76,14 +73,9 @@ class IdeGen(JvmToolTaskMixin, Task):
              help='Includes scala sources in the project; otherwise compiles them and adds them '
                   'to the project classpath.')
     register('--use-source-root', action='store_true', default=False,
-             help='Use source_root() settings to collapse sourcepaths in project and determine '
+             help='Use source roots to collapse sourcepaths in project and determine '
                   'which paths are used for tests.  This is usually what you want if your repo '
                   ' uses a maven style directory layout.')
-    register('--infer-test-from-siblings', action='store_true',
-             help='When determining if a path should be added to the IDE, check to see if any of '
-                  'its sibling source_root() entries define test targets.  This is usually what '
-                  'you want so that resource directories under test source roots are picked up as '
-                  'test paths.')
     register('--debug_port', type=int, default=5005,
              help='Port to use for launching tasks under the debugger.')
     register('--source-jars', action='store_true', default=True,
@@ -112,20 +104,12 @@ class IdeGen(JvmToolTaskMixin, Task):
       round_manager.require('java')
     if options.scala:
       round_manager.require('scala')
-    # TODO(Garrett Malmquist): Clean this up by using IvyUtils in the caller, passing it confs as
-    # the parameter. See John's comments on RB 716.
-    round_manager.require_data('ivy_jar_products')
-    round_manager.require('jar_dependencies')
-    round_manager.require('jar_map_default')
-    if options.source_jars:
-      round_manager.require('jar_map_sources')
-    if options.javadoc_jars:
-      round_manager.require('jar_map_javadoc')
 
   class Error(TaskError):
     """IdeGen Error."""
 
   class TargetUtil(object):
+
     def __init__(self, context):
       self.context = context
 
@@ -172,6 +156,21 @@ class IdeGen(JvmToolTaskMixin, Task):
     self.intransitive = self.get_options().intransitive
     self.debug_port = self.get_options().debug_port
 
+  def resolve_jars(self, targets):
+    executor = self.create_java_executor()
+    confs = ['default']
+    if self.get_options().source_jars:
+      confs.append('sources')
+    if self.get_options().javadoc_jars:
+      confs.append('javadoc')
+    compile_classpath = ClasspathProducts(self.get_options().pants_workdir)
+    self.resolve(executor=executor,
+                 targets=targets,
+                 classpath_products=compile_classpath,
+                 confs=confs,
+                 extra_args=())
+    return compile_classpath
+
   def _prepare_project(self):
     targets, self._project = self.configure_project(
         self.context.targets(),
@@ -180,7 +179,8 @@ class IdeGen(JvmToolTaskMixin, Task):
     self.configure_compile_context(targets)
 
   def configure_project(self, targets, debug_port):
-    jvm_targets = [t for t in targets if t.has_label('jvm') or t.has_label('java')]
+    jvm_targets = [t for t in targets if t.has_label('jvm') or t.has_label('java') or
+                   isinstance(t, Resources)]
     if self.intransitive:
       jvm_targets = set(self.context.target_roots).intersection(jvm_targets)
     project = Project(self.project_name,
@@ -190,6 +190,7 @@ class IdeGen(JvmToolTaskMixin, Task):
                       self.use_source_root,
                       get_buildroot(),
                       debug_port,
+                      self.context,
                       jvm_targets,
                       not self.intransitive,
                       self.TargetUtil(self.context),
@@ -226,11 +227,12 @@ class IdeGen(JvmToolTaskMixin, Task):
     jars = OrderedSet()
     excludes = OrderedSet()
     compiles = OrderedSet()
+
     def prune(target):
       if target.is_jvm:
         if target.excludes:
           excludes.update(target.excludes)
-        jars.update(jar for jar in target.jar_dependencies if jar.rev)
+        jars.update(jar for jar in target.jar_dependencies)
         if is_cp(target):
           target.walk(compiles.add)
 
@@ -280,16 +282,7 @@ class IdeGen(JvmToolTaskMixin, Task):
 
           self._project.internal_jars.add(ClasspathEntry(cp_jar, source_jar=cp_source_jar))
 
-  def copy_jars(self, jar_list, dest_dir):
-    cp_jars = []
-    if jar_list:
-      for jar in jar_list:
-        cp_jar = os.path.join(dest_dir, os.path.basename(jar))
-        shutil.copy(jar, cp_jar)
-        cp_jars.append(cp_jar)
-    return cp_jars
-
-  def map_external_jars(self):
+  def map_external_jars(self, targets):
     external_jar_dir = os.path.join(self.gen_project_workdir, 'external-libs')
     safe_mkdir(external_jar_dir, clean=True)
 
@@ -298,31 +291,44 @@ class IdeGen(JvmToolTaskMixin, Task):
 
     external_javadoc_jar_dir = os.path.join(self.gen_project_workdir, 'external-libjavadoc')
     safe_mkdir(external_javadoc_jar_dir, clean=True)
-    jar_products = self.context.products.get_data('ivy_jar_products')
-    jar_paths = get_jar_infos(jar_products, confs=['default', 'sources', 'javadoc'])
-    for entry in jar_paths.values():
-      binary_jars = entry.get('default')
-      sources_jars = entry.get('sources')
-      javadoc_jars = entry.get('javadoc')
-      cp_jars = self.copy_jars(binary_jars, external_jar_dir)
-      cp_source_jars = self.copy_jars(sources_jars, external_source_jar_dir)
-      cp_javadoc_jars = self.copy_jars(javadoc_jars, external_javadoc_jar_dir)
-      for i in range(len(cp_jars)):
-        cp_jar = cp_jars[i]
-        if i < len(cp_source_jars):
-          cp_source_jar = cp_source_jars[i]
-        else:
-          cp_source_jar = None
-        if i < len(cp_javadoc_jars):
-          cp_javadoc_jar = cp_javadoc_jars[i]
-        else:
-          cp_javadoc_jar = None
-        self._project.external_jars.add(ClasspathEntry(cp_jar,
-                                                       source_jar=cp_source_jar,
-                                                       javadoc_jar=cp_javadoc_jar))
+
+    classpath_products = self.resolve_jars(targets) or ClasspathProducts(self.get_options().pants_workdir)
+    cp_entry_by_classifier_by_orgname = defaultdict(lambda: defaultdict(dict))
+    for conf, jar_entry in classpath_products.get_artifact_classpath_entries_for_targets(targets):
+      coord = (jar_entry.coordinate.org, jar_entry.coordinate.name)
+      classifier = jar_entry.coordinate.classifier
+      cp_entry_by_classifier_by_orgname[coord][classifier] = jar_entry
+
+    def copy_jar(cp_entry, dest_dir):
+      if not cp_entry:
+        return None
+      cp_jar = os.path.join(dest_dir, os.path.basename(cp_entry.path))
+      shutil.copy(cp_entry.path, cp_jar)
+      return cp_jar
+
+    # Per org.name (aka maven "project"), collect the primary artifact and any extra classified
+    # artifacts, taking special note of 'sources' and 'javadoc' artifacts that IDEs handle specially
+    # to provide source browsing and javadocs for 3rdparty libs.
+    for cp_entry_by_classifier in cp_entry_by_classifier_by_orgname.values():
+      primary_jar = copy_jar(cp_entry_by_classifier.pop(None, None), external_jar_dir)
+      sources_jar = copy_jar(cp_entry_by_classifier.pop('sources', None), external_source_jar_dir)
+      javadoc_jar = copy_jar(cp_entry_by_classifier.pop('javadoc', None), external_javadoc_jar_dir)
+      if primary_jar:
+        self._project.external_jars.add(ClasspathEntry(jar=primary_jar,
+                                                       source_jar=sources_jar,
+                                                       javadoc_jar=javadoc_jar))
+
+      # Treat all other jars as opaque with no source or javadoc attachments of their own.  An
+      # example are jars with the 'tests' classifier.
+      for jar_entry in cp_entry_by_classifier.values():
+        extra_jar = copy_jar(jar_entry, external_jar_dir)
+        self._project.external_jars.add(ClasspathEntry(extra_jar))
 
   def execute(self):
     """Stages IDE project artifacts to a project directory and generates IDE configuration files."""
+    # Grab the targets in-play before the context is replaced by `self._prepare_project()` below.
+    targets = self.context.targets()
+
     self._prepare_project()
 
     if self.context.options.is_known_scope('compile.checkstyle'):
@@ -336,9 +342,8 @@ class IdeGen(JvmToolTaskMixin, Task):
       scalac_classpath = self.tool_classpath('scalac', scope='scala-platform')
 
     self._project.set_tool_classpaths(checkstyle_classpath, scalac_classpath)
-    targets = self.context.targets()
     self.map_internal_jars(targets)
-    self.map_external_jars()
+    self.map_external_jars(targets)
 
     idefile = self.generate_project(self._project)
     if idefile:
@@ -360,21 +365,21 @@ class ClasspathEntry(object):
 class SourceSet(object):
   """Models a set of source files."""
 
-  def __init__(self, root_dir, source_base, path, is_test, content_type=''):
+  def __init__(self, root_dir, source_base, path, is_test=False, resources_only=False):
     """
     :param string root_dir: full path to the root of the project containing this source set
     :param string source_base: the relative path from root_dir to the base of this source set
     :param string path: relative path from the source_base to the base of the sources in this set
-    :param bool is_test: true iff the sources contained by this set implement test cases
-    :param string content_type: Content type resources or test resources for scala/java project
+    :param bool is_test: true if the sources contained by this set implement test cases
+    :param bool resources_only: true if a target has resources but no sources.
     """
 
     self.root_dir = root_dir
     self.source_base = source_base
     self.path = path
     self.is_test = is_test
+    self.resources_only = resources_only
     self._excludes = []
-    self.content_type = content_type
 
   @property
   def excludes(self):
@@ -398,6 +403,15 @@ class SourceSet(object):
   def __hash__(self):
     return hash(self._key_tuple)
 
+  def __repr__(self):
+    return "root_dir={} source_base={} path={} is_test={} resources_only={} _excludes={}".format(
+      self.root_dir,
+      self.source_base,
+      self.path,
+      self.is_test,
+      self.resources_only,
+      self._excludes)
+
 
 class Project(object):
   """Models a generic IDE project that is comprised of a set of BUILD targets."""
@@ -412,7 +426,7 @@ class Project(object):
         yield ext
 
   @staticmethod
-  def _collapse_by_source_root(source_sets):
+  def _collapse_by_source_root(source_roots, source_sets):
     """Collapse SourceSets with common source roots into one SourceSet instance.
 
     Use the registered source roots to collapse all source paths under a root.
@@ -420,33 +434,33 @@ class Project(object):
     a test path.  This method will give unpredictable results if source root entries overlap.
 
     :param list source_sets: SourceSets to analyze
-    :returns: list of SourceSets collapsed to the source root paths.
+    :returns: list of SourceSets collapsed to the source root paths.  There may be duplicate
+      entries in this list which will be removed by dedup_sources()
     """
-
-    roots_found = set()  # remember the roots we've already encountered
     collapsed_source_sets = []
     for source in source_sets:
       query = os.path.join(source.source_base, source.path)
-      source_root = SourceRoot.find_by_path(query)
+      source_root = source_roots.find_by_path(query)
       if not source_root:
         collapsed_source_sets.append(source)
-      elif not source_root in roots_found:
-        roots_found.add(source_root)
-        collapsed_source_sets.append(SourceSet(source.root_dir, source_root, "", source.is_test))
+      else:
+        collapsed_source_sets.append(SourceSet(source.root_dir, source_root.path, "",
+                                               is_test=source.is_test,
+                                               resources_only=source.resources_only))
     return collapsed_source_sets
 
   def __init__(self, name, has_python, skip_java, skip_scala, use_source_root, root_dir,
-               debug_port, targets, transitive, target_util, spec_excludes):
+               debug_port, context, targets, transitive, target_util, spec_excludes):
     """Creates a new, unconfigured, Project based at root_dir and comprised of the sources visible
     to the given targets."""
-
+    self.context = context
     self.target_util = target_util
     self.name = name
     self.root_dir = root_dir
     self.targets = OrderedSet(targets)
     self.transitive = transitive
 
-    self.sources = set()
+    self.sources = []
     self.py_sources = []
     self.py_libs = []
     self.resource_extensions = set()
@@ -464,13 +478,36 @@ class Project(object):
     self.external_jars = OrderedSet()
     self.spec_excludes = spec_excludes
 
-  def configure_python(self, source_roots, test_roots, lib_roots):
-    self.py_sources.extend(SourceSet(get_buildroot(), root, None, False) for root in source_roots)
-    self.py_sources.extend(SourceSet(get_buildroot(), root, None, True) for root in test_roots)
-    for root in lib_roots:
+  def configure_python(self, source_paths, test_paths, lib_paths):
+    self.py_sources.extend(SourceSet(get_buildroot(), root, None) for root in source_paths)
+    self.py_sources.extend(SourceSet(get_buildroot(), root, None, is_test=True) for root in test_paths)
+    for root in lib_paths:
       for path in os.listdir(os.path.join(get_buildroot(), root)):
         if os.path.isdir(os.path.join(get_buildroot(), root, path)) or path.endswith('.egg'):
-          self.py_libs.append(SourceSet(get_buildroot(), root, path, False))
+          self.py_libs.append(SourceSet(get_buildroot(), root, path, is_test=False))
+
+  @classmethod
+  def dedup_sources(cls, source_set_list):
+    """Remove duplicate source sets from the source_set_list.
+
+      Sometimes two targets with the same path are added to the source set. Remove duplicates
+      with the following rules:
+
+      1) If two targets are resources_only with different settings for is_test, is_test = False
+      2) If the targets have different settings for resources_only, resources_only = False
+      3) If the two non-resource-only targets have different settings for is_test, is_test = True
+    """
+    deduped_sources = set(filter(lambda s: not s.resources_only and s.is_test,
+                                 source_set_list))
+    deduped_sources.update(filter(lambda s: not s.resources_only,
+                                  source_set_list))
+    deduped_sources.update(filter(lambda s : s.resources_only and not s.is_test,
+                                  source_set_list))
+    deduped_sources.update(filter(lambda s : s.resources_only and s.is_test,
+                                  source_set_list))
+
+    # re-sort the list, makes the generated project easier to read.
+    return sorted(list(deduped_sources))
 
   def configure_jvm(self, extra_source_paths, extra_test_paths):
     """
@@ -482,7 +519,7 @@ class Project(object):
     # TODO(John Sirois): much waste lies here, revisit structuring for more readable and efficient
     # construction of source sets and excludes ... and add a test!
 
-    analyzed = OrderedSet()
+    analyzed_targets = OrderedSet()
     targeted = set()
 
     def relative_sources(target):
@@ -496,7 +533,7 @@ class Project(object):
                not (self.skip_scala and is_scala(target))))
       return result
 
-    def configure_source_sets(relative_base, sources, is_test, content_type=''):
+    def configure_source_sets(relative_base, sources, is_test=False, resources_only=False):
       absolute_base = os.path.join(self.root_dir, relative_base)
       paths = set([os.path.dirname(source) for source in sources])
       for path in paths:
@@ -504,15 +541,9 @@ class Project(object):
         # Note, this can add duplicate source paths to self.sources().  We'll de-dup them later,
         # because we want to prefer test paths.
         targeted.add(absolute_path)
-        source_set = SourceSet(self.root_dir, relative_base, path, is_test, content_type)
-        if source_set in self.sources and content_type:
-         # Note, same resource can be added twice.
-         # 1. Once with the content_type attached with target.has_resources loop
-         # 2. Second without content_type from command line target.
-         # We do not want to skip resources in flow 2 as we can have resources on the command line
-         # which are not attached to any target.
-         self.sources.remove(source_set)
-        self.sources.add(source_set)
+        source_set = SourceSet(self.root_dir, relative_base, path,
+                               is_test=is_test, resources_only=resources_only)
+        self.sources.append(source_set)
 
     def find_source_basedirs(target):
       dirs = set()
@@ -523,8 +554,8 @@ class Project(object):
       return dirs
 
     def configure_target(target):
-      if target not in analyzed:
-        analyzed.add(target)
+      if target not in analyzed_targets:
+        analyzed_targets.add(target)
         self.has_scala = not self.skip_scala and (self.has_scala or is_scala(target))
 
         # Hack for java_sources and Eclipse/IntelliJ: add java_sources to project
@@ -532,24 +563,22 @@ class Project(object):
           for java_source in target.java_sources:
             configure_target(java_source)
 
+        # Resources are already in the target set
         if target.has_resources:
           resources_by_basedir = defaultdict(set)
           for resources in target.resources:
+            analyzed_targets.add(resources)
             resources_by_basedir[resources.target_base].update(relative_sources(resources))
           for basedir, resources in resources_by_basedir.items():
             self.resource_extensions.update(Project.extract_resource_extensions(resources))
-            if target.is_test:
-              configure_source_sets(basedir, resources, is_test=target.is_test,
-                                    content_type=IdeGen.TEST_RESOURCES)
-            else:
-              configure_source_sets(basedir, resources, is_test=target.is_test,
-                                    content_type=IdeGen.RESOURCES)
-
+            configure_source_sets(basedir, resources, is_test=target.is_test,
+                                  resources_only=True)
         if target.has_sources():
           test = target.is_test
           self.has_tests = self.has_tests or test
           base = target.target_base
-          configure_source_sets(base, relative_sources(target), is_test=test)
+          configure_source_sets(base, relative_sources(target), is_test=test,
+                                resources_only=isinstance(target, Resources))
 
         # TODO(Garrett Malmquist): This is dead code, and should be redone/reintegrated.
         # Other BUILD files may specify sources in the same directory as this target. Those BUILD
@@ -557,8 +586,8 @@ class Project(object):
         # this target globs children as well.  Gather all these candidate BUILD files to test for
         # sources they own that live in the directories this targets sources live in.
         target_dirset = find_source_basedirs(target)
-        if target.address.is_synthetic:
-          return [] # Siblings don't make sense for synthetic addresses.
+        if not isinstance(target.address, BuildFileAddress):
+          return []  # Siblings only make sense for BUILD files.
         candidates = self.target_util.get_all_addresses(target.address.build_file)
         for ancestor in target.address.build_file.ancestors():
           candidates.update(self.target_util.get_all_addresses(ancestor))
@@ -566,29 +595,28 @@ class Project(object):
           candidates.update(self.target_util.get_all_addresses(sibling))
         for descendant in target.address.build_file.descendants(spec_excludes=self.spec_excludes):
           candidates.update(self.target_util.get_all_addresses(descendant))
+
         def is_sibling(target):
           return source_target(target) and target_dirset.intersection(find_source_basedirs(target))
 
         return filter(is_sibling, [self.target_util.get(a) for a in candidates if a != target.address])
 
+    resource_targets = []
     for target in self.targets:
-      target.walk(configure_target, predicate=source_target)
+      if isinstance(target, Resources):
+        # Wait to process these until all resources that are reachable from other targets are
+        # processed.  That way we'll only add a new SourceSet if this target has never been seen
+        # before. This allows test resource SourceSets to be properly keep the is_test property.
+        resource_targets.append(target)
+      else:
+        target.walk(configure_target, predicate=source_target)
+
+    for target in resource_targets:
+      target.walk(configure_target)
 
     def full_path(source_set):
       return os.path.join(source_set.root_dir, source_set.source_base, source_set.path)
 
-    def dedup_sources(source_set_list):
-      """Sometimes two targets with the same path are added to the source set. One is a target where
-       is_test evaluates to True and the other were it evaluates to False.  When this happens,
-       make sure we prefer the SourceSet with is_test set to True.
-      """
-      deduped_sources = set(filter(lambda source_set: source_set.is_test, source_set_list))
-      for source_set in source_set_list:
-        if not source_set.is_test and source_set not in deduped_sources:
-          deduped_sources.add(source_set)
-      # re-sort the list, makes the generated project easier to read.
-      return sorted(list(deduped_sources))
-
     # Check if there are any overlapping source_sets, and output an error message if so.
     # Overlapping source_sets cause serious problems with package name inference.
     overlap_error = ('SourceSets {current} and {previous} evaluate to the same full path.'
@@ -643,12 +671,12 @@ class Project(object):
     targets = OrderedSet()
     for target in self.targets:
       target.walk(lambda target: targets.add(target), source_target)
-    targets.update(analyzed - targets)
-    self.sources.update(SourceSet(get_buildroot(), p, None, False) for p in extra_source_paths)
-    self.sources.update(SourceSet(get_buildroot(), p, None, True) for p in extra_test_paths)
+    targets.update(analyzed_targets - targets)
+    self.sources.extend(SourceSet(get_buildroot(), p, None, is_test=False) for p in extra_source_paths)
+    self.sources.extend(SourceSet(get_buildroot(), p, None, is_test=True) for p in extra_test_paths)
     if self.use_source_root:
-      self.sources = Project._collapse_by_source_root(self.sources)
-    self.sources = dedup_sources(self.sources)
+      self.sources = Project._collapse_by_source_root(self.context.source_roots, self.sources)
+    self.sources = self.dedup_sources(self.sources)
 
     return targets
 
