@@ -24,12 +24,15 @@ from pants.backend.python.python_requirement import PythonRequirement
 from pants.backend.python.python_setup import PythonRepos, PythonSetup
 from pants.backend.python.targets.python_tests import PythonTests
 from pants.backend.python.tasks.python_task import PythonTask
+from pants.base.build_environment import get_buildroot
 from pants.base.exceptions import TaskError, TestFailedTaskError
-from pants.base.target import Target
-from pants.base.workunit import WorkUnit
+from pants.base.workunit import WorkUnitLabel
+from pants.build_graph.target import Target
+from pants.task.testrunner_task_mixin import TestRunnerTaskMixin
 from pants.util.contextutil import (environment_as, temporary_dir, temporary_file,
                                     temporary_file_path)
 from pants.util.dirutil import safe_mkdir, safe_open
+from pants.util.process_handler import SubprocessProcessHandler
 from pants.util.strutil import safe_shlex_split
 
 
@@ -66,13 +69,15 @@ class PythonTestResult(object):
     return self._failed_targets
 
 
-class PytestRun(PythonTask):
+class PytestRun(TestRunnerTaskMixin, PythonTask):
   _TESTING_TARGETS = [
     # Note: the requirement restrictions on pytest and pytest-cov match those in requirements.txt,
     # to avoid confusion when debugging pants tests.
     # TODO: make these an option, so any pants install base can pick their pytest version.
     PythonRequirement('pytest>=2.6,<2.7'),
-    PythonRequirement('pytest-timeout'),
+    # NB, pytest-timeout 1.0.0 introduces a conflicting pytest>=2.8.0 requirement, see:
+    #   https://github.com/pantsbuild/pants/issues/2566
+    PythonRequirement('pytest-timeout<1.0.0'),
     PythonRequirement('pytest-cov>=1.8,<1.9'),
     PythonRequirement('unittest2', version_filter=lambda py, pl: py.startswith('2')),
     PythonRequirement('unittest2py3k', version_filter=lambda py, pl: py.startswith('3'))
@@ -112,15 +117,24 @@ class PytestRun(PythonTask):
   def supports_passthru_args(cls):
     return True
 
-  def execute(self):
-    def is_python_test(target):
+  def __init__(self, *args, **kwargs):
+    super(PytestRun, self).__init__(*args, **kwargs)
+
+  def _test_target_filter(self):
+    def target_filter(target):
       return isinstance(target, PythonTests)
 
-    test_targets = list(filter(is_python_test, self.context.targets()))
+    return target_filter
+
+  def _validate_target(self, target):
+    pass
+
+  def _execute(self, all_targets):
+    test_targets = self._get_test_targets()
     if test_targets:
       self.context.release_lock()
       with self.context.new_workunit(name='run',
-                                     labels=[WorkUnit.TOOL, WorkUnit.TEST]) as workunit:
+                                     labels=[WorkUnitLabel.TOOL, WorkUnitLabel.TEST]) as workunit:
         # pytest uses py.io.terminalwriter for output. That class detects the terminal
         # width and attempts to use all of it. However we capture and indent the console
         # output, leading to weird-looking line wraps. So we trick the detection code
@@ -139,11 +153,10 @@ class PytestRun(PythonTask):
       # Coverage often throws errors despite tests succeeding, so force failsoft in that case.
       fail_hard = not self.get_options().fail_slow and not self.get_options().coverage
       for target in targets:
-        if isinstance(target, PythonTests):
-          rv = self._do_run_tests([target], workunit)
-          results[target] = rv
-          if not rv.success and fail_hard:
-            break
+        rv = self._do_run_tests([target], workunit)
+        results[target] = rv
+        if not rv.success and fail_hard:
+          break
 
       for target in sorted(results):
         self.context.log.info('{0:80}.....{1:>10}'.format(target.id, str(results[target])))
@@ -405,19 +418,19 @@ class PytestRun(PythonTask):
     pex_info = PexInfo.default()
     pex_info.entry_point = 'pytest'
 
-    with self.cached_chroot(interpreter=interpreter,
-                            pex_info=pex_info,
-                            targets=targets,
-                            platforms=('current',),
-                            extra_requirements=self._TESTING_TARGETS) as chroot:
-      pex = chroot.pex()
-      with self._maybe_shard() as shard_args:
-        with self._maybe_emit_junit_xml(targets) as junit_args:
-          with self._maybe_emit_coverage_data(targets,
-                                              chroot.path(),
-                                              pex,
-                                              workunit) as coverage_args:
-            yield pex, shard_args + junit_args + coverage_args
+    chroot = self.cached_chroot(interpreter=interpreter,
+                                pex_info=pex_info,
+                                targets=targets,
+                                platforms=('current',),
+                                extra_requirements=self._TESTING_TARGETS)
+    pex = chroot.pex()
+    with self._maybe_shard() as shard_args:
+      with self._maybe_emit_junit_xml(targets) as junit_args:
+        with self._maybe_emit_coverage_data(targets,
+                                            chroot.path(),
+                                            pex,
+                                            workunit) as coverage_args:
+          yield pex, shard_args + junit_args + coverage_args
 
   def _do_run_tests_with_args(self, pex, workunit, args):
     try:
@@ -431,18 +444,28 @@ class PytestRun(PythonTask):
       }
       profile = self.get_options().profile
       if profile:
-        env['PEX_PROFILE'] = '{0}.subprocess.{1:.6f}'.format(profile, time.time())
+        env['PEX_PROFILE_FILENAME'] = '{0}.subprocess.{1:.6f}'.format(profile, time.time())
       with environment_as(**env):
-        rc = self._pex_run(pex, workunit, args=args, setsid=True)
+        rc = self._spawn_and_wait(pex, workunit, args=args, setsid=True)
         return PythonTestResult.rc(rc)
+    except TestFailedTaskError:
+      # _spawn_and_wait wraps the test runner in a timeout, so it could
+      # fail with a TestFailedTaskError. We can't just set PythonTestResult
+      # to a failure because the resultslog doesn't have all the failures
+      # when tests are killed with a timeout. Therefore we need to re-raise
+      # here.
+      raise
     except Exception:
       self.context.log.error('Failed to run test!')
       self.context.log.info(traceback.format_exc())
       return PythonTestResult.exception()
 
-  # Pattern for lines such as:
+  # Pattern for lines such as ones below.  The second one is from a test inside a class.
   # F testprojects/tests/python/pants/constants_only/test_fail.py::test_boom
-  RESULTLOG_FAILED_PATTERN = re.compile(r'F +(.+)::(.+)')
+  # F testprojects/tests/python/pants/constants_only/test_fail.py::TestClassName::test_boom
+
+  # 'E' is here as well to catch test errors, not just test failures.
+  RESULTLOG_FAILED_PATTERN = re.compile(r'[EF] +(.+?)::(.+)')
 
   @classmethod
   def _get_failed_targets_from_resultlogs(cls, filename, targets):
@@ -490,7 +513,10 @@ class PytestRun(PythonTask):
         failed_targets = self._get_failed_targets_from_resultlogs(resultlog_path, targets)
         return result.with_failed_targets(failed_targets)
 
-      args = []
+      # N.B. the `--confcutdir` here instructs pytest to stop scanning for conftest.py files at the
+      # top of the buildroot. This prevents conftest.py files from outside (e.g. in users home dirs)
+      # from leaking into pants test runs. See: https://github.com/pantsbuild/pants/issues/2726
+      args = ['--confcutdir', get_buildroot()]
       if self._debug:
         args.extend(['-s'])
       if self.get_options().colors:
@@ -511,11 +537,17 @@ class PytestRun(PythonTask):
           return run_and_analyze(resultlog_path)
 
   def _pex_run(self, pex, workunit, args, setsid=False):
+    process = self._spawn(pex, workunit, args, setsid=False)
+    return process.wait()
+
+  def _spawn(self, pex, workunit, args, setsid=False):
     # NB: We don't use pex.run(...) here since it makes a point of running in a clean environment,
     # scrubbing all `PEX_*` environment overrides and we use overrides when running pexes in this
     # task.
+
     process = subprocess.Popen(pex.cmdline(args),
                                preexec_fn=os.setsid if setsid else None,
                                stdout=workunit.output('stdout'),
                                stderr=workunit.output('stderr'))
-    return process.wait()
+
+    return SubprocessProcessHandler(process)
