@@ -16,3 +16,15 @@ def atomic_copy(src, dst):
   with temporary_file(root_dir=os.path.dirname(dst)) as tmp_dst:
     shutil.copyfile(src, tmp_dst.name)
     os.rename(tmp_dst.name, dst)
+
+
+def create_size_estimators():
+  def line_count(filename):
+    with open(filename, 'rb') as fh:
+      return sum(1 for line in fh)
+  return {
+    'linecount': lambda srcs: sum(line_count(src) for src in srcs),
+    'filecount': lambda srcs: len(srcs),
+    'filesize': lambda srcs: sum(os.path.getsize(src) for src in srcs),
+    'nosize': lambda srcs: 0,
+  }
